/* ***** BEGIN LICENSE BLOCK *****
 * Licensed under Version: MPL 1.1/GPL 2.0/LGPL 2.1
 * Full Terms at http://mozile.mozdev.org/license.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Playsophy code.
 *
 * The Initial Developer of the Original Code is Playsophy
 * Portions created by the Initial Developer are Copyright (C) 2002-2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *		Karl Guertin <grayrest@grayrest.com>
 *		James A. Overton <james@overton.ca>
 *		Christian Stocker (Bitflux)
 *		Paul Everitt
 *		David Palm
 *
 * ***** END LICENSE BLOCK ***** */

/**
@fileoverview

<p>DESCRIPTION:<br />
The code here is responsible for detecting when Mozile should be active.
It gets called a lot, so mozileDetect() has to be very fast.
*/

function mozileGetCurrentTarget() {
	return mozileTarget;
}
/**
Note: this and the startup value should be the same. 
*/
function mozileClearCurrentTarget() {
	mozileTarget = null;
}

/**
Check all element nodes in the Document element for the contentEditable attribute.
<p>When found, and contentEditable="true" add Mozile (in-line) style attributes to the element node.
<p>Only works if the Document element is <html> or <HTML>
*/
function mozileConvertCE() {
	var f = ["mozileDetection.js","mozileConvertCE"];
	var target, count, l, i;
	try {
		target = document.commandDispatcher.focusedWindow;
		if ( target.toString() == "[object ChromeWindow]" ) {
			return null;
		}
		count = 0;
		if ( target.document.documentElement.tagName.toLowerCase() == "html" ) {
			l = target.document.documentElement.getElementsByTagName("*");
			for( i=0; i<l.length; i++ ) {
				if ( l[i].hasAttributes() && l[i].hasAttribute( "contentEditable" ) ) {
					if ( l[i].getAttribute( "contentEditable" ).toLowerCase() == "true" ) {
						l[i].style.MozUserModify = "read-write";
						l[i].style.MozUserInput = "enabled";
						l[i].style.MozUserSelect = "text"; //Is this needed?
						++count;
					}
				}
			}
			return mozile.strings.getFormattedString("CEconvert", [count]);
		}
		else {
			return mozile.strings.getString("CEnothtml");
		}
	}
	catch (e) {
		mozileDebug(f,1,"Exception: "+e);
		return null;
	}
}

function mozileMakeDocumentEditable() {
	var f = ["mozileDetection.js","mozileMakeDocumentEditable"];
	var result = mozileMakeEditable();
	if ( result === null ) {
		result = mozile.strings.getString("Edit.error2");
	}
	mozileDebug(f,3,result);
	mozileDialog(result);
}

/**
Make any html/xml document editable.

<p>Adds the Mozile style tags to the first <body> element if the document root is html.
<p>Adds a processing instruction to link a CSS file for other documents.
<p>Note: If you then save the document, none of this info is removed.
<p>If the document is already Mozile aware it won't be made editable.
<p>Note: all sorts of non-html/xml files can be made "editable" when they shouldn't be.
Doesn't do any harm, but would be better if the tests could be improved.
*/
function mozileMakeEditable() {
	var f = ["mozileDetection.js","mozileMakeEditable"];
	var target, l;
	try {
		target = document.commandDispatcher.focusedWindow;
		if ( target.toString() == "[object ChromeWindow]" ) {
			return null;
		}
		if ( ("moziled" in target.document) && target.document.moziled==true ) {
			return mozile.strings.getString("Edit.moziled");
		}
		if ( target.document.documentElement.tagName.toLowerCase() == "html" ) {
			l = target.document.documentElement.getElementsByTagName("body");
			if ( l.length > 0 ) {
				l[0].style.MozUserModify = "read-write";
				l[0].style.MozUserInput = "enabled";
				l[0].style.MozUserSelect = "text";
				return mozile.strings.getString("Edit.done");
			} else { /*Browser will add body to an html doc if it is missing!*/
				return mozile.strings.getString("Edit.no.body");
			}
		} else {
			if ( "xmlVersion" in target.document ) {
			//If some sort of XML document ... [expand this test as needed]
				l = target.document.documentElement;
				var ss = target.document.styleSheets
				if(ss.length==0){
				//If there are no stylesheets, create a processing instruction and insert it.
					var pi = target.document.createProcessingInstruction("xml-stylesheet",'href="chrome://mozile/content/mes/edit-XHTML.css" type="text/css"');
					target.document.insertBefore(pi,l);
					ss = target.document.styleSheets;
				}
				//Add the editing CSS to the first style sheet.
				ss[0].insertRule(l.localName + " { -moz-user-modify: read-write; -moz-user-input: enabled; -moz-user-select: text; }", ss[0].cssRules.length);
				return mozile.strings.getString("Edit.done");
			} else {
				return mozile.strings.getString("Edit.not.xml");
			}
		}
	}
	catch (e) {
		mozileDebug(f,1,"Exception: "+e);
		return null;
	}
}
/**
Toggle Mozile detect and edit ON/OFF.
*/
function mozileToggleOnOff() {
	if (mozilePrefs.getBoolPref("mozile.onBool")) {
		mozileSwitchOff();
		mozilePrefs.setBoolPref("mozile.onBool",false);
		return false;
	}else{
		mozileSwitchOn();
		mozilePrefs.setBoolPref("mozile.onBool",true);
		return true;
	}
}

/**
Switch on Mozile by adding the mouse and load listeners.
*/
function mozileSwitchOn() {
	mozileAddLoadListener();
	mozileAddMouseListener();
	mozileToolbarSwitchOn();
	mozileDetect();
}
/**
Switch off Mozile by removing the mouse and load listeners.
Note: unload, beforeunload and keypress listeners are not removed.
*/
function mozileSwitchOff() {
	mozileRemoveLoadListener();
	mozileRemoveMouseListener();
	mozileNoDetect();
	mozileToolbarSwitchOff();
	mozileRestoreCaret();
//	mozileNoDetect();
}

function mozileAddLoadListener() {
document.addEventListener("load", mozileLoadScan, true);
}
function mozileRemoveLoadListener() {
document.removeEventListener("load", mozileLoadScan, true);
}
/**
Scan documents on load for the Mozile wake-up meta tag.
If the wake-up tag is found the toolbar is switched on.
All scanned documents are marked.
*/
function mozileLoadScan(event) {
	try{
		if ( event.originalTarget instanceof HTMLDocument ) {
			var doc=event.originalTarget;
			if ( !("moziled" in doc) ) {
				doc.moziled=false;
				var meta = doc.getElementsByTagName("meta");
				for (var i=0; i<meta.length; i++) {
					if( meta[i].hasAttribute("name") && meta[i].getAttribute("name")=="mozile" ) {
						doc.moziled=true;
						break;
					} 
				}
			}	
			if(doc.moziled){
				mozileInject(doc.defaultView);
				mozileTools.show();
				mozileClearDisplay();
			}else if(!mozileTools.toolbar.collapsed && mozilePrefs.getBoolPref('mozile.autohideBool')) {
				mozileTools.hide();
			}
		}
	}catch(e){}
}


/**
Tidy up routine if detection lost.
*/
function mozileNoDetect(){
	if(arguments.length==0 || arguments[0]==true){
		mozile.lastContextNode=null;
		mozileTarget=mozileSelection=null;
		mozileClearDisplay();
	}
 	mozileRestoreCaret();
	return 0;
}

/**
Mozile Detect.
Check to see if the user has selected an editable area.
If so, set the global mozileTarget and mozileSelection variables.
Load Mozile javascript files if not already loaded.
If not, return 0, and don't do anything else.
*/
function mozileDetect() {
	mozileKeyboard.clearRefresh();
	try {
		// get the focused window
		var mozileCheckTarget = document.commandDispatcher.focusedWindow;
		//Ignore if in browser.
		if (mozileCheckTarget.toString() == "[object ChromeWindow]") {return 0}
	} catch (e) {return mozileNoDetect()}
	
	try{
		var f = ["mozileDetection.js","mozileDetect"];
		//Get a reference to the selection object of the focused window.
		var mozileCheckSelection = mozileCheckTarget.getSelection();

		// if there's no node, Mozile won't be able to edit
		try {
  		if ( mozileCheckSelection.anchorNode == null ) {
				throw "anchorNode==null"; }
  		if ( mozileCheckSelection.anchorNode.parentNode == null ) {
				throw "parentNode==null"; }
			var currentContextNode = mozileGetContext(mozileCheckSelection);
			if ( currentContextNode == null ) {
				throw "null return from mozileGetContext"; }
		} catch(e) {
			mozileDebug(f,3,"No node to edit: "+e);
			return mozileNoDetect();
		}
		mozileDebug(f,3,"Detecting node: "+currentContextNode.tagName);

// MAX - detection doesn't get this far. Some very strange browser behavour for the Selection object
// when these have focus. All sorts of phantom nodes are generated that (unsurpisingly) cause eDOM to
// throw a wobbly. Maybe some of the internal workings of the input element are getting mixed up with
// the selected object???
//			// don't edit these nodes!			
//		switch (currentContextNode.tagName.toLowerCase()) {
//			case "input" :
//			case "textarea" :
//				return mozileNoDetect(mozileIsLoaded(mozileCheckTarget));
//		}

			// is it the same (editable) node?
			//This is a short circuit to save executing the code below...
		try{
			if(mozile.lastContextNode === currentContextNode) {
				mozileDebug(f,3,"We're still inside the same editable node");
				return 1;
			}
		} catch(e) { 
			throw e;
		} 

		// Check if the selection is editable.
		var CSSenabled = mozileCheckTarget.document.defaultView.getComputedStyle(currentContextNode, null).MozUserModify;
//		mozileDebug(f,3,"CSS Enabled? "+CSSenabled);
		if(CSSenabled!="read-write") {return mozileNoDetect()}

		// Load eDOM if not already loaded.
		var inject=false;
		if( mozileIsLoaded(mozileCheckTarget) === false ) {
			mozileInject(mozileCheckTarget);
			//There can be problems just after injection.
			//Make sure a range is not selected.
			mozileCheckSelection.collapseToStart();

			//Injection of Javascript files takes time ... causes Display update to fail because
			//eDOM routine used by active toolbar check is not present. Hence, add a timeout refresh.
			//Increase the delay if this is too short.
			mozileCheckTarget.setTimeout(mozileUpdateDisplay,400);
			inject=true;
		}
		mozileSetLoaded( mozileCheckTarget, "true" );
		//Set global variables	
		mozileTarget = mozileCheckTarget;
		mozileSelection = mozileCheckSelection;
		mozile.lastContextNode = currentContextNode;
		if (!inject) {
			mozileUpdateDisplay();
		}
		mozileDebug(f,3,"Detect done.");
		return 1;
	}
	catch(e) { 
		mozileDebug(f,0,"Detect Died: "+e);
		return mozileNoDetect();
	}
}
