/* ***** BEGIN LICENSE BLOCK *****
 * Licensed under Version: MPL 1.1/GPL 2.0/LGPL 2.1
 * Full Terms at http://mozile.mozdev.org/license.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Playsophy code.
 *
 * The Initial Developer of the Original Code is Playsophy
 * Portions created by the Initial Developer are Copyright (C) 2002-2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *	Karl Guertin <grayrest@grayrest.com>
 *	James A. Overton <james@overton.ca>
 *	Christian Stocker (Bitflux)
 *	Paul Everitt
 *	David Palm
 *
 * ***** END LICENSE BLOCK ***** */

/**
@fileoverview
Mozile Main Javascript.

Global variables and initialisation functions.
*/

/* === Global Variables === */

//Mozile Constants
var MOZILEC = new Object();			//Global object for constants.
MOZILEC.EXTENSION_DIR = "chrome://mozile/content/eDOM/";
MOZILEC.MES_DEFAULT = "chrome://mozile/content/mes/mes-XHTML-en-US.xml";
MOZILEC.TAGS_XHTML = "chrome://mozile/content/mes/tags-XHTML.css";
MOZILEC.HOME = "http://mozile.mozdev.org/";
MOZILEC.HELP = "chrome://mozile/content/docs/index.html";
MOZILEC.DEMO = "chrome://mozile/content/demo/mozileDemo.html";
MOZILEC.NAMESPACE_XHTML = "http://www.w3.org/1999/xhtml";

var mozileTarget = "";         // the currently focussed window, if it has loaded Mozile.
var mozileSelection = "";      // the eDOM selection of the currently focussed window.
var mozilePrefs = "";          // Netscape preferences component

var mozile = new Object();     //Global object.
mozile.startup=true;
mozile.lastContextNode = null; //The last detected node.
mozile.keyMax = 20;            // the number of key presses before the state is saved to the Undo Stack.
mozile.undoMax = 100;          // the number of undo steps available.

var mozileSourceSplitter = ""; // part of the Mozile source view box
var mozileSourceBox = "";      // the Mozile source view box
var mozileSourceTextbox = "";  // the Mozile source textbox
var mozileSourceMode = 1;      // source editing mode: 1=block, 0=page
var mozileSourceError = "";    // contains the source of the last parser error message
var mozileSourceSync = "";     // track the mozileSourceSync checkbox element
var mozileSourceNamespace = ""; // track the mozileSourceNamespace checkbox element

/* === Inline code to execute on startup === */

	document.addEventListener("load", mozileStartup, true);

/* === Startup Functions === */

/**
Initialise toolbar variables.
Note: can't have onload handler for overlay file, hence it must be done this way. 
*/
function mozileStartup() {
	mozilePrefsInit();
	mozileTools.init();
	mozile.strings=document.getElementById("mozile-strings");
	document.removeEventListener("load", mozileStartup, true);
	if ( mozilePrefs.getBoolPref("mozile.onBool") ) {
		mozileSwitchOn();
	} else {
		mozileSwitchOff();
	}
	mozileDetect();
	mozile.startup=false;
}

/**
Initializes the mozilePrefs variable.
The catch blocks set the initial values when Mozile is first installed.
*/
function mozilePrefsInit() {

	// the caret requires added privileges
	netscape.security.PrivilegeManager.enablePrivilege('UniversalXPConnect');
	mozilePrefs = Components.classes['@mozilla.org/preferences-service;1'].getService(Components.interfaces.nsIPrefService).getBranch(null);	
	
	try {
		mozilePrefs.getBoolPref("mozile.acceleratorBool");
	} catch(e) {
		mozilePrefs.setBoolPref("mozile.acceleratorBool",false);
	}
	
	try {
		mozilePrefs.getBoolPref("mozile.autohideBool");
	} catch(e) {
		mozilePrefs.setBoolPref("mozile.autohideBool",false);
	}
	
	try {
		mozilePrefs.getBoolPref("mozile.caretAlwaysOnBool");
	} catch(e) {
		mozilePrefs.setBoolPref("mozile.caretAlwaysOnBool",false);
	}
	
	try {
		mozilePrefs.getBoolPref("mozile.onBool");
	} catch(e) {
		mozilePrefs.setBoolPref("mozile.onBool",true);
	}
	
	try {
		mozilePrefs.getBoolPref("mozile.promptOnUnloadBool");
	} catch(e) {
		mozilePrefs.setBoolPref("mozile.promptOnUnloadBool",true);
	}
	
	try {
		mozilePrefs.getIntPref("mozile.debugLevelInt");
	} catch(e) {
		mozilePrefs.setIntPref("mozile.debugLevelInt",0);
	}
	
	try {
		mozilePrefs.getCharPref("mozile.saveContentString");
	} catch(e) {
		mozilePrefs.setCharPref("mozile.saveContentString","page");
	}
	
	try {
		mozilePrefs.getCharPref("mozile.saveMethodString");
	} catch(e) {
		mozilePrefs.setCharPref("mozile.saveMethodString","display");
	}
	
	try {
		mozilePrefs.getCharPref("mozile.saveHttpPathString");
	} catch(e) {
		mozilePrefs.setCharPref("mozile.saveHttpPathString","");
	}
}
